; File Name   :	kim1\TINY3 low version 0000 0200.BIN
; Base Address:	$0000 Range: $0000 - $0B00 Loaded length: $0B00

; Tom Pittman's 6502 tiny BASIC
; reverse analyzed from dump kim1\TINY3 low version 0000 0200.BIN
; from http://retro.hansotten.nl/6502-sbc/kim-1-manuals-and-software/kim-1-software/
; http://retro.hansotten.nl/uploads/files/TinyBasicKIMbinary.zip
;
; assemble with arnold macro cross assembler (http://john.ccac.rwth-aachen.de:8000/as/)
; Volker Pohlers 07-2020, 100% identically binary code to dump

;------------------------------------------------------------------------------
;AS-Funktionen
hi              function x,(x>>8)&255
lo              function x, x&255


		cpu	6502

		org    	0

; page 0 variables, stack etc.
; 100h break routine
; 200h-ACAh tiny basic (less than 2k!)


		org    0

		rmb    32
start_prgm:     rmb    2            ; start of BASIC text (0x900)
end_ram:        rmb    2            ; end of available RAM
end_prgm:       rmb    2            ; end of BASIC text
top_of_stack:   rmb    2            ; top of return stack pointer location
basic_lineno:   rmb    2            ; save for current line number to be executed
il_pc_reg:      rmb    2            ; program counter for IL code
basic_ptr:      rmb    2            ; pointer to currently executed BASIC byte
basicptr_save:  rmb    2            ; temporary save for basic_ptr
expr_stack:     rmb    80           ; Input line & Expression stack
				    ; lowest byte of expr_stack (0x30)
rnd_seed:       rmb    2            ; used as seed value for RND function
				    ; note this is actually top of predecrementing expr_stack (0x80)
var_tbl:        rmb    2*26         ; variables (A-Z), 26 words
LS_end:         rmb    2            ; used to store addr of end of LS listing,
				    ; start of list is in basic_ptr
BP_save:        rmb    2            ; another temporary save for basic_ptr
X_save:         rmb    2            ; temporary save for X
IL_temp:        rmb    2            ; temporary for various IL operations
				    ; used for branch to IL handler routine for opcode
run_mode:       rmb    1            ; run mode
				    ; = 0 direct mode
				    ; <> 0 running program
column_cnt:     rmb    1            ; counter for output columns (required for TAB in PRINT)
				    ; if bit 7 is set, suppress output (XOFF)
expr_stack_top: rmb    1            ; low byte of expr_stack_top (used in 8 bit comparisons)
expr_stack_x:   rmb    1            ; high byte of expr_stack_top (==0x00, used with X register)
word_C2:	rmb    2
il_pc_save:     rmb    2            ; save of IL program counter
il_sp_reg:	rmb    2



		org	$100

; test for break from input device, set	=1 if break
break:		LDA	$1740
		CLC
		BMI	break3
break1:		LDA	$1740
		BPL	break1
		JSR	break2
break2:		LDA	#$0FF
		JSR	out_v
		SEC
break3:		RTS


;******************************************************************************

		org	$0200

; cold start vector
		JMP	cold_s		; Do cold start initialization

; warm start vector
		JMP	warm_s		; do warm start

; vector: get a character from input device into A
; jump to system specific input routine
in_v:		JMP	$1E5A

; print a character in A to output device
; jump to system specific output routine
out_v:		JMP	$1EA0

; test for break from input device, set C=1 if break
; jump to $100
bv:		JMP	break

; some standard constants
BSC:            fcb    $5F          ; backspace code (should be 0x7f, but actually is '_')
LSC:            fcb    $18          ; line cancel code (CTRL-X)
PCC:            fcb    $0           ; CRLF padding characters
				    ; low 7 bits are number of NUL/0xFF
				    ; bit7=1: send 0xFF, =0, send NUL
TMC:            fcb    $80          ;
SSS:            fcb    $4           ; reserved bytes at end_prgm (to prevent return stack
				    ; underflow (spare area)

;******************************************************************************
; utility routines for BASIC (not called in interpreter code)
;******************************************************************************

PEEK:		STX	word_C2+1
		BCC	loc_21D
		STX	word_C2+1
		STA	(word_C2),Y
		RTS
loc_21D:	LDA	(word_C2),Y
loc_21F:	LDY	#0
		RTS

;******************************************************************************
; Interpreter jump table
;******************************************************************************

il_jumptable:
	       FDB IL_BBR    ; 0x40-0x5f: backward branch
	       FDB IL_FBR    ; 0x60-0x7f: forward_branch
	       FDB IL_BC     ; 0x80-0x9f: string match branch
	       FDB IL_BV     ; 0xa0-0xbf: branch if not variable
	       FDB IL_BN     ; 0xc0-0xdf: branch if not number
	       FDB IL_BE     ; 0xe0-0xff: branch if not eoln
	       FDB IL_NO     ; 0x08:      no operation
	       FDB IL_LB     ; 0x09:      push literal byte to TOS
	       FDB IL_LN     ; 0x0a:      push literal word to TOS
	       FDB IL_DS     ; 0x0b:      duplicate stack top
	       FDB IL_SP     ; 0x0c:      pop TOS into A:B
	       FDB pop_byte  ; 0x0d:      undocumented: pop byte
	       FDB IL_NO     ; 0x0e:      no operation
	       FDB IL_NO     ; 0x0f:      no operation
	       FDB IL_SB     ; 0x10:      save BASIC pointer
	       FDB IL_RB     ; 0x11:      restore BASIC pointer
	       FDB IL_FV     ; 0x12:      fetch variable
	       FDB IL_SV     ; 0x13:      store variable
	       FDB IL_GS     ; 0x14:      save GOSUB line
	       FDB IL_RS     ; 0x15:      restore saved line
	       FDB IL_GO     ; 0x16:      goto
	       FDB IL_NE     ; 0x17:      negate
	       FDB IL_AD     ; 0x18:      add
	       FDB IL_SU     ; 0x19:      subtract
	       FDB IL_MP     ; 0x1a:      multiply
	       FDB IL_DV     ; 0x1b:      divide
	       FDB IL_CP     ; 0x1c:      compare
	       FDB IL_NX     ; 0x1d:      next BASIC statement
	       FDB IL_NO     ; 0x1e:      reserved
	       FDB IL_LS     ; 0x1f:      list program
	       FDB IL_PN     ; 0x20:      print number
	       FDB IL_PQ     ; 0x21:      print BASIC string
	       FDB IL_PT     ; 0x22:      print tab
	       FDB IL_NL     ; 0x23:      new line
	       FDB IL_PC     ; 0x24:      print literal string
	       FDB IL_NO     ; 0x25:      undocumented op for SPC(x) function
	       FDB IL_NO     ; 0x26:      reserved
	       FDB IL_GL     ; 0x27:      get input line
	       FDB ILRES1    ; 0x28:      reserved
	       FDB ILRES2    ; 0x29:      reserved
	       FDB IL_IL     ; 0x2A:      Insert BASIC line
	       FDB IL_MT     ; 0x2B:      mark BASIC program space empty
	       FDB IL_XQ     ; 0x2C:      execute
	       FDB warm_s    ; 0x2D:      warm start
	       FDB IL_US     ; 0x2E:      machine language subroutine call
	       FDB IL_RT     ; 0x2F:      IL subroutine return


err_at:		FCB " AT ", $80

IL_baseaddr:	FDB ILTBL

;------------------------------------------------------------------------------
; Cold start entry point
;------------------------------------------------------------------------------

cold_s:		LDA	#0		; initialize start of BASIC
		STA	start_prgm	; $0B00
		STA	end_ram
		LDA	#$0B
		STA	start_prgm+1
		STA	end_ram+1
		LDY	#1
find_end_ram:	LDA	(end_ram),Y
		TAX
		EOR	#$0FF		; complement
		STA	(end_ram),Y
		CMP	(end_ram),Y
		PHP
		TXA
		STA	(end_ram),Y
		INC	end_ram
		BNE	loc_2A6
		INC	end_ram+1
loc_2A6:	PLP
		BEQ	find_end_ram
		DEY			; Y should be $00 now

;------------------------------------------------------------------------------
; IL instruction MT: clear program
;------------------------------------------------------------------------------

IL_MT:		CLD
		LDA	start_prgm	; load start area
		ADC	SSS		; add spare area after end of program
		STA	end_prgm	; save as end of program
		TYA			; Retrieve Y
		ADC	start_prgm+1
		STA	end_prgm+1
		TYA			; Retrieve Y
		STA	(start_prgm),Y	; Store A in the first byte of program memory
		INY			; Increment Y
		STA	(start_prgm),Y	; Store A in the second byte of program memory

;------------------------------------------------------------------------------
; warm start entry point
;------------------------------------------------------------------------------

warm_s:		LDA	end_ram		; set return stack to end of RAM
		STA	il_sp_reg
		STA	top_of_stack
		LDA	end_ram+1
		STA	il_sp_reg+1
		STA	top_of_stack+1
		JSR	crlf		; emit a CRLF

; enters here to start IL loop;
; return here after error stop
restart_il:	LDA	IL_baseaddr	; load pointer to IL
		STA	il_pc_reg
		LDA	IL_baseaddr+1
		STA	il_pc_reg+1
		LDA	#rnd_seed
		STA	expr_stack_x	; set expr_stack top to 0x0080
		LDA	#expr_stack
		STA	expr_stack_top	; set expr_stack_low = 0x30
		LDX	#0
		STX	run_mode	; set run_mode = 0 (no program)
		STX	word_C2
		DEX
		TXS

il_mainloop:	CLD
		JSR	fetch_il_op	; fetch next IL opcode
		JSR	exec_il_opcode	; execute current IL instruction
		JMP	il_mainloop	; next instruction

; unknown (old code or a hidden serial number??)
		FCB    2
		FCB  $53

;------------------------------------------------------------------------------
; with IL opcode in A, decode opcode and
; branch to appropriate	routine
;------------------------------------------------------------------------------

exec_il_opcode:	CMP	#$30 		; is opcode below 0x30?
		BCS	handle_30_ff	; no, skip to handler for higher opcodes
		CMP	#8		; is it below 8?
		BCC	handle_il_SX	; yes, skip to handler for SX instructions
		ASL	A		; make word index
		TAX

loc_2FC:	LDA	loc_21F,X	; load handler address via offset
		PHA
		LDA	loc_21D+1,X
		PHA
		PHP			; push addres on stack
		RTI			; jump to handler via RTI

; handle SX instruction
handle_il_SX:	ADC	expr_stack_x
		TAX
		LDA	(expr_stack_x),Y
		PHA
		LDA	0,X
		STA	(expr_stack_x),Y
		PLA
		STA	0,X
		RTS

;------------------------------------------------------------------------------
; common error routine
;------------------------------------------------------------------------------

error:		JSR	crlf		; emit CRLF
		LDA	#'!'
		JSR	out_v		; emit exclamation mark
		LDA	il_pc_reg	; load IL PC
		SEC
		SBC	IL_baseaddr	; subtract origin of interpreter
		TAX
		LDA	il_pc_reg+1
		SBC	IL_baseaddr+1
		JSR	emit_number	; print instruction of IL
		LDA	run_mode	; is not in program?
		BEQ	error_no_lineno	; no, suppress output of line number
		LDA	#lo(err_at)
		STA	il_pc_reg
		LDA	#hi(err_at)
		STA	il_pc_reg+1
		JSR	IL_PC		; print string at il_prgm_cnt, i.e. "AT "
		LDX	basic_lineno	; get line number
		LDA	basic_lineno+1
		JSR	emit_number	; print it

error_no_lineno:
		LDA	#7
		JSR	out_v		; emit BEL (0x07) character
		JSR	crlf		; emit CRLF

;---------------------------------------

restart_il_nocr:
		LDA	top_of_stack
		STA	il_sp_reg
		LDA	top_of_stack+1
		STA	il_sp_reg+1
		JMP	restart_il

;---------------------------------------

expr_check_4bytes:
		LDX	#(rnd_seed-4)	; $7C

expr_check_nbytes:
		CPX	expr_stack_x

loc_358:	BCC	error
		LDX	expr_stack_x
		INC	expr_stack_x
		INC	expr_stack_x
		CLC
		RTS

;------------------------------------------------------------------------------
; long branch instruction
;------------------------------------------------------------------------------

IL_BBR:		DEC	IL_temp+1	; adjust position for negative jump (effectively 2's complement)

IL_FBR:		LDA	IL_temp+1	; test new position high byte
		BEQ	error		; was displacement 0?
					; yes, this is an error condition

il_goto:	LDA	IL_temp		; get new IL target address
		STA	il_pc_reg	; do the jump
		LDA	IL_temp+1
		STA	il_pc_reg+1
		RTS

;------------------------------------------------------------------------------
; part of interpreter loop: handle opcode 0x30-3f
;------------------------------------------------------------------------------

handle_30_ff:	CMP	#$40		; above or equal 0x40?
		BCS	handle_40_ff	; yes, handle elsewhere

		; handle the J/JS instructions
		PHA			; save opcode
		JSR	fetch_il_op	; get next byte of instruction (low address)
		ADC	IL_baseaddr	; add to IL interpreter base
		STA	IL_temp
		PLA			; restore opcode
		PHA			; save for later
		AND	#7		; mask out addressbits
		ADC	IL_baseaddr+1	; add to base address
		STA	IL_temp+1	; save in temporary
		PLA			; restore opcode
		AND	#8		; mask J/JS bit
		BNE	il_goto		; if set, is GOTO
		LDA	IL_temp		; save in temporary
		LDX	il_pc_reg	; get current IL PC
		STA	il_pc_reg	; save new IL PC
		STX	IL_temp		; save old in temporary
		LDA	IL_temp+1
		LDX	il_pc_reg+1
		STA	il_pc_reg+1
		STX	IL_temp+1

handle_30a:	LDA	il_sp_reg
		SBC	#1
		STA	il_sp_reg
		BCS	handle_30b
		DEC	il_sp_reg+1

handle_30b:	CMP	end_prgm
		LDA	il_sp_reg+1
		SBC	end_prgm+1
		BCC	loc_358
		LDA	IL_temp
		STA	(il_sp_reg),Y
		INY
		LDA	IL_temp+1
		STA	(il_sp_reg),Y
		RTS

;------------------------------------------------------------------------------
; handle the opcodes >=0x40
;------------------------------------------------------------------------------

handle_40_ff:	PHA			; save opcode for later
		LSR	A		; get opcode high nibble
		LSR	A
		LSR	A
		LSR	A
		AND	#$0E		; make 0x04,0x06,...0x0e
		TAX
		PLA			; restore opcode
		CMP	#$60 		; is it BBR?
		AND	#$1F		; mask out displacement bits
		BCS	not_bbr		; was not backward branch
		ORA	#$E0 		; make displacement negative

not_bbr:	CLC
		BEQ	displ_error	; displacement is zero? yes, skip
		ADC	il_pc_reg	; add displayement to current IL_PC
		STA	IL_temp
		TYA
		ADC	il_pc_reg+1

displ_error:	STA	IL_temp+1	; store high byte of new address
					; if displayement=0, store high byte=0
					; -> invalid IL address, will lead to error
		JMP	loc_2FC		; jump to handler routine

;------------------------------------------------------------------------------
; IL instruction string	match branch
; jump forward if string was not matched
;------------------------------------------------------------------------------

IL_BC:		LDA	basic_ptr	; save pointer to current BASIC character
		STA	BP_save
		LDA	basic_ptr+1
		STA	BP_save+1

bc_loop:	JSR	get_nchar	; skip spaces
		JSR	fetch_basicchar	; consume char
		EOR	(il_pc_reg),Y
		TAX
		JSR	fetch_il_op
		TXA
		BEQ	bc_loop
		ASL	A
		BEQ	locret_404

		LDA	BP_save		; restore BASIC pointer
		STA	basic_ptr
		LDA	BP_save+1
		STA	basic_ptr+1

j_FBR:		JMP	IL_FBR		; and branch forward

;------------------------------------------------------------------------------
; IL instruction: jump if not end of line
;------------------------------------------------------------------------------

IL_BE:		JSR	get_nchar	; get current BASIC char
		CMP	#$0D		; is it a CR?
		BNE	j_FBR		; no, jump forward

locret_404:	RTS			; continue

;------------------------------------------------------------------------------
; IL instruction: branch if not	variable
; if variable, push 2*ASCII to expr_stack
; (0x41..0x5A => 0x82...0xB4
;  == offset to	var table into zero page)
;------------------------------------------------------------------------------

IL_BV:		JSR	get_nchar	; get current BASIC char
		CMP	#$5B 		;'Z'+1
		BCS	j_FBR		; no, jump forward
		CMP	#'A'		; is it an alphanumeric?
		BCC	j_FBR		; no, jump forward
		ASL	A		; yes, double the ASCII code
					; (make it a word index into var table
		JSR	expr_push_byte	; push it on the stack
					; ...and consume this character
					; (fall thru to fetch_basicchar)

;------------------------------------------------------------------------------
; get next BASIC char from program or line
; return in A, Z=1 if CR
;------------------------------------------------------------------------------

fetch_basicchar:
		LDY	#0		; get address of current BASIC byte
		LDA	(basic_ptr),Y	; get byte
		INC	basic_ptr	; advance to next character
		BNE	loc_41E
		INC	basic_ptr+1

loc_41E:	CMP	#$0D		; check if 0x0d (end of line)
		CLC
		RTS

;------------------------------------------------------------------------------
; get next BASIC char (without advance)
; C=1 if digit
;------------------------------------------------------------------------------

get_nchar0:	JSR	fetch_basicchar	; get next char

get_nchar:	LDA	(basic_ptr),Y
		CMP	#' '		; is it a space?
		BEQ	get_nchar0	; yes, skip that
		CMP	#$3A ; '9'+1	; return C=1 if number
		CLC
		BPL	get_nchar1
		CMP	#'0'		; is it a digit?
get_nchar1:	RTS

;------------------------------------------------------------------------------
; IL instruction: branch if not	number
; if digit, convert this and following digits to number
; and push on expr_stack
;------------------------------------------------------------------------------

IL_BN:		JSR	get_nchar	; get BASIC char
		BCC	j_FBR		; if not digit, do forward branch
		STY	IL_temp		; clear temporary for number
		STY	IL_temp+1

loop_bn:	LDA	IL_temp		; multiply TEMP by 10
		LDX	IL_temp+1
		ASL	IL_temp		; temp*2
		ROL	IL_temp+1
		ASL	IL_temp		; temp*4
		ROL	IL_temp+1
		CLC
		ADC	IL_temp		; temp*5
		STA	IL_temp
		TXA
		ADC	IL_temp+1
		ASL	IL_temp		; temp*10
		ROL	A
		STA	IL_temp+1
		JSR	fetch_basicchar	; get and consume this char
		AND	#$0F		; mask out low nibble (0...9)
		ADC	IL_temp		; add into temp
		STA	IL_temp
		TYA
		ADC	IL_temp+1
		STA	IL_temp+1
		JSR	get_nchar	; get next char
		BCS	loop_bn		; loop as long as digit found
		JMP	expr_push_word	; push on expr_stack

;------------------------------------------------------------------------------
; find BASIC line whose lineno is on stack
; discard from stack
; return found line in basic_ptr
; Z=1 if line is matched exactly
;------------------------------------------------------------------------------

find_line:	JSR	IL_SP		; pop word
		LDA	IL_temp		; save in temporary
		ORA	IL_temp+1	; check if zero (invalid)
		BEQ	loc_4BC		; if so, error

		; find BASIC line whose lineno is in IL_temp
find_line1:	LDA	start_prgm	; set BASIC pointer to start
		STA	basic_ptr
		LDA	start_prgm+1
		STA	basic_ptr+1

test_line:	JSR	save_lineno	; save current lineno
		BEQ	find_exit	; if zero, skip to end
		LDA	basic_lineno	; compare line number with current line
		CMP	IL_temp
		LDA	basic_lineno+1
		SBC	IL_temp+1
		BCS	find_exit	; if above, exit

find_eoln:	JSR	fetch_basicchar	; get next char
		BNE	find_eoln	; not CR? loop
		JMP	test_line	; check next line

find_exit:	LDA	basic_lineno	; compare current linenumber with searched one
		EOR	IL_temp
		BNE	find_exit1
		LDA	basic_lineno+1
		EOR	IL_temp+1
find_exit1:	RTS

;------------------------------------------------------------------------------
; IL instruction print string
;------------------------------------------------------------------------------

pc_loop:	JSR	emit_char	; emit a character and continue
					; with PC instruction

IL_PC:		JSR	fetch_il_op	; get next byte of instruction
		BPL	pc_loop	 	; if positive, skip

emit_char:	INC	column_cnt
		BMI	emit_char1
		JMP	out_v		 ; if last char, emit it and terminate
emit_char1:	DEC	column_cnt
emit_char2:	RTS

;------------------------------------------------------------------------------
; IL instruction PQ
;------------------------------------------------------------------------------

loop_pq:	CMP	#'"'		; is character string terminator?
		BEQ	emit_char2	; yes, exit
		JSR	emit_char	; otherwise emit char
					; and redo PQ instruction

IL_PQ:		JSR	fetch_basicchar	; get next char from BASIC text
		BNE	loop_pq		; if not CR, loop

loc_4BC:	JMP	error		; error - unterminated string

;------------------------------------------------------------------------------
;  IL instruction print	tab
;------------------------------------------------------------------------------

IL_PT:		LDA	#' '		; emit a space
		JSR	emit_char
		LDA	column_cnt
		AND	#$87
		BMI	emit_char2
		BNE	IL_PT
		RTS

;------------------------------------------------------------------------------
; IL instruction compare
; stack: TOS, MASK, NOS
; compare TOS with NOS
; MASK is bit0 = less
;      bit1 = equal
;      bit2 = greater
; if compare result AND mask return <>0, next IL op is skipped
;
;           bit 210  
; 1 < 		001
; 2 =		010
; 3 <=		011	
; 4 >		100
; 5 <> oder ><	101
; 6 >=		110
;------------------------------------------------------------------------------

IL_CP:		LDX	#(rnd_seed-5)	; $7B
		JSR	expr_check_nbytes
		INC	expr_stack_x
		INC	expr_stack_x
		INC	expr_stack_x
		SEC
		LDA	3,X
		SBC	0,X
		STA	0,X
		LDA	4,X
		SBC	1,X
		BVC	loc_4E9
		EOR	#$80
		ORA	#1

loc_4E9:	BMI	cp_is_lt	; if less, skip
		BNE	cp_is_gt	; if greater, skip
		ORA	0,X		; is result 0?
		BEQ	cp_is_eq

cp_is_gt:	LSR	2,X		; shift bit 2 into carry

cp_is_eq:	LSR	2,X		; shift bit 1 into carry

cp_is_lt:	LSR	2,X		; shift bit 0 into carry
		BCC	locret_505	; not matched: exit, continue new IL op

;------------------------------------------------------------------------------
; get the next IL opcode and increment the IL PC
;------------------------------------------------------------------------------

fetch_il_op:	LDY	#0		; get IL PC
		LDA	(il_pc_reg),Y	; read next opcode
		INC	il_pc_reg
		BNE	loc_503
		INC	il_pc_reg+1	; advance to next byte
loc_503:	ORA	#0
locret_505:	RTS

;------------------------------------------------------------------------------
; IL instruction: advance to next BASIC	line
;------------------------------------------------------------------------------

IL_NX:		LDA	run_mode	; run mode = 0?
		BEQ	loc_532		; yes, continue program

nx_loop:	JSR	fetch_basicchar	; get char from program
		BNE	nx_loop		; is not CR, loop
		JSR	save_lineno	; store line number
		BEQ	j1_error	; is 0000 (end of program) -> error

;------------------------------------------------------------------------------
; enters here from a GOTO,
; basic	pointer	points to new line
;------------------------------------------------------------------------------

go_found_line:	JSR	do_runmode	; set run mode = running
		JSR	bv		; test for BREAK
		BCS	do_break	; if C=1, do break
		LDA	il_pc_save	; restore IL_PC which was saved in XQ or GO
		STA	il_pc_reg
		LDA	il_pc_save+1
		STA	il_pc_reg+1
		RTS

do_break:	LDA	IL_baseaddr	; restart interpreter
		STA	il_pc_reg
		LDA	IL_baseaddr+1
		STA	il_pc_reg+1

j1_error:	JMP	error		 ; and emit break error

;------------------------------------------------------------------------------
; fragment of code for IL_NX
;------------------------------------------------------------------------------

loc_532:	STA	column_cnt
		JMP	restart_il_nocr

;------------------------------------------------------------------------------
; IL instruction: execute program
;------------------------------------------------------------------------------

IL_XQ:		LDA	start_prgm	; set current start of program
		STA	basic_ptr
		LDA	start_prgm+1
		STA	basic_ptr+1
		JSR	save_lineno	; save current line number
		BEQ	j1_error	; if zero, error
		LDA	il_pc_reg	; save current IL_PC
		STA	il_pc_save
		LDA	il_pc_reg+1
		STA	il_pc_save+1

do_runmode:	LDA	#1		; runmode=1
		STA	run_mode
		RTS

;------------------------------------------------------------------------------
; IL instruction GO
;------------------------------------------------------------------------------

IL_GO:		JSR	find_line	; find line which lineno is on stack
		BEQ	go_found_line	; found? yes, skip

go_error:	LDA	IL_temp		; set requested lineno as current
		STA	basic_lineno
		LDA	IL_temp+1
		STA	basic_lineno+1
		JMP	error		; error - line not found

;------------------------------------------------------------------------------
; IL instruction: restore saved	line
;------------------------------------------------------------------------------

IL_RS:		JSR	loc_8FD		; get saved line 2 levels off stack
		JSR	loc_8F4
		JSR	find_line1	; find the basic line
		BNE	go_error	; line not found? -> error
		RTS

;------------------------------------------------------------------------------
; save current linenumber
;------------------------------------------------------------------------------

save_lineno:	JSR	fetch_basicchar	; get char from program code
		STA	basic_lineno	; save as high lineno
		JSR	fetch_basicchar	; get char from program code
		STA	basic_lineno+1	; save as low lineno
		ORA	basic_lineno	; Check 0
		RTS

;------------------------------------------------------------------------------
; IL instruction: duplicate top	   of expr_stack
;------------------------------------------------------------------------------

IL_DS:		JSR	IL_SP		; pop top of expr_stack
		JSR	expr_push_word	; push twice on expr_stack
					; (fall through to expr_push_word routine)

;------------------------------------------------------------------------------
; push IL_temp word on expr_stack
;------------------------------------------------------------------------------

expr_push_word:
		LDA	IL_temp+1

loc_582:	JSR	expr_push_byte
		LDA	IL_temp

;------------------------------------------------------------------------------
; push byte on expr_stack
;------------------------------------------------------------------------------

expr_push_byte:
		LDX	expr_stack_x	; get expr_stack top
		DEX			; make space for another byte
		STA	0,X		; save A as new TOS (top of stack value)
		STX	expr_stack_x	; set new stack top
		CPX	expr_stack_top	; stack overflow?
		BNE	IL_NO		; no, exit

j_error:	JMP	error		; error: stack overflow

pop_byte:	LDX	expr_stack_x	; get address of stack top
		CPX	#rnd_seed	; is it > 0x80?
		BPL	j_error		; yes, stack underflow error
		LDA	0,X		; get TOS byte
		INC	expr_stack_x

IL_NO:		RTS

;------------------------------------------------------------------------------
; emit number
;------------------------------------------------------------------------------

emit_number:	STA	IL_temp+1
		STX	IL_temp
		JMP	loc_5B8

IL_PN:		LDX	expr_stack_x	; get address of stack top
		LDA	1,X		; is number negative?
		BPL	loc_5B5		; no, skip
		JSR	IL_NE		; negate top of stack
		LDA	#'-'
		JSR	emit_char

loc_5B5:	JSR	IL_SP		; pop TOS

loc_5B8:	LDA	#$1F
		STA	BP_save
		STA	X_save
		LDA	#$2A ; '*'
		STA	BP_save+1
		STA	X_save+1
		LDX	IL_temp
		LDY	IL_temp+1
		SEC

loop_10000s:	INC	BP_save
		TXA
		SBC	#$10		; subtract 10000 (0x2710) until negative
		TAX
		TYA
		SBC	#$27
		TAY
		BCS	loop_10000s

loop_1000s:	DEC	BP_save+1	; is now negative value, subtract until positive again
		TXA
		ADC	#$0E8 		; add 1000 (0x3e8) until positive again
		TAX
		TYA
		ADC	#3
		TAY
		BCC	loop_1000s
		TXA

loc_5E2:	SEC

loop_100s:	INC	X_save		; is positive value now
		SBC	#$64		; subtract 100 (0x54) until negative
		BCS	loop_100s
		DEY
		BPL	loc_5E2

loop_10s:	DEC	X_save+1	; is now negative
		ADC	#$0A		; add 10 until positive again
		BCC	loop_10s	; counter for 10's become 0x10..0x19
					; B contains remianing 1's digits

		ORA	#'0'		; suppress leading zeroes
		STA	IL_temp
		LDA	#' '
		STA	IL_temp+1
		LDX	#$0FB

loc_5FC:	STX	word_C2+1
		LDA	IL_temp+1,X
		ORA	IL_temp+1
		CMP	#' '
		BEQ	loc_60F
		LDY	#'0'
		STY	IL_temp+1
		ORA	IL_temp+1	; make it a real ASCII '0'...'9'
		JSR	emit_char	; and print it

loc_60F:	LDX	word_C2+1
		INX
		BNE	loc_5FC
		RTS

;------------------------------------------------------------------------------
; IL Instruction List BASIC source
;------------------------------------------------------------------------------

IL_LS:		LDA	basic_ptr+1	; save current BASIC pointer
		PHA
		LDA	basic_ptr
		PHA
		LDA	start_prgm	; default start: begin of program
		STA	basic_ptr
		LDA	start_prgm+1
		STA	basic_ptr+1
		LDA	end_prgm	; default end: load X with end of program
		LDX	end_prgm+1
		JSR	ls_getlineno	; if argument to list given, make this new end
					; note "LIST start,end", so the first result
					; popped off stack is the end
		BEQ	ls_nostart	; no more argument on stack
		JSR	ls_getlineno	; save first position in LS_begin
					; get another argument into basic_ptr, if any

ls_nostart:	LDA	basic_ptr	; compare start and end of listing
		SEC
		SBC	LS_end
		LDA	basic_ptr+1
		SBC	LS_end+1
		BCS	ls_exit		; start > end? yes, exit: nothing (more) to list
		JSR	save_lineno	; save lineno of current line
		BEQ	ls_exit		; is end of program (line 0)? yes, exit
		LDX	basic_lineno
		LDA	basic_lineno+1
		JSR	emit_number	; print line number
		LDA	#' '		; print a space

ls_loop:	JSR	emit_char
		JSR	bv		; check for break
		BCS	ls_exit		; if break, exit
		JSR	fetch_basicchar	; get next char from line
		BNE	ls_loop		; if not CR, loop output
		JSR	IL_NL		; emit a CRLF
		JMP	ls_nostart	; loop with next line

;------------------------------------------------------------------------------
; called with an address into BASIC code
; return Z=1 if	no argument
;------------------------------------------------------------------------------

ls_getlineno:	STA	LS_end		; store as default end of listing
		INC	LS_end
		BNE	loc_662
		INX			; increment X
loc_662:	STX	LS_end+1	; store as default end of listing
		LDY	expr_stack_x	; get expr_stack ptr
		CPY	#rnd_seed	; is stack empty?
		BEQ	locret_682	; yes, no arg given...done
		JSR	find_line	; find the line (after the lineno) that was given on
					; stack (start line number)
					; result in X=basic_ptr

ls_to_linestart:
		LDA	basic_ptr	; point back to lineno that was found
		LDX	basic_ptr+1
		SEC
		SBC	#2
		BCS	loc_677
		DEX

loc_677:	STA	basic_ptr
		JMP	loc_948

ls_exit:	PLA			; restore old BASIC pointer
		STA	basic_ptr
		PLA
		STA	basic_ptr+1

locret_682:	RTS

;------------------------------------------------------------------------------
; IL instruction: emit new line
;------------------------------------------------------------------------------

IL_NL:		LDA	column_cnt	; if column > 127, suppress output
		BMI	locret_682

;------------------------------------------------------------------------------
; do a CRLF
;------------------------------------------------------------------------------

crlf:		LDA	#$0D		; emit carriage return character
		JSR	out_v
		LDA	PCC		; get padding count
		AND	#$7F
		STA	column_cnt
		BEQ	loc_69C
loc_695:	JSR	emit_nul_padding	; emit padding
		DEC	column_cnt
		BNE	loc_695
loc_69C:	LDA	#$0A		; emit line feed character
		JMP	out_v_padding

;---------------------------------------

do_xon:		LDY	TMC		; get XOFF flag

do_xoff:	STY	column_cnt
		BCS	gl_loop

;------------------------------------------------------------------------------
; IL instruction: get input line
; uses expr_stack as buffer
;------------------------------------------------------------------------------

IL_GL:		LDA	#expr_stack	; store floor of expr_stack as BASIC pointer
		STA	basic_ptr
		STA	expr_stack_top
		STY	basic_ptr+1
		JSR	expr_push_word	; save for later (may be variable address, or alike)

gl_loop:	EOR	rnd_seed	; use random A to create some entropy
		STA	rnd_seed
		JSR	in_v		; get a char from input device
		LDY	#0
		LDX	expr_stack_top
		AND	#$7F		; make 7bit ASCII
		BEQ	gl_loop		; if NUL, ignore
		CMP	#$7F		; if 0xFF/0x7F, ignore
		BEQ	gl_loop
		CMP	#$13		; if DC3 (XOFF) handle XOFF
		BEQ	do_xoff
		CMP	#$0A		; if LF, done
		BEQ	do_xon
		CMP	LSC		; line cancel?
		BEQ	gl_ctrlx
		CMP	BSC		; is it "_" ? (back character)
		BNE	gl_chkend	; no, skip
		CPX	#'0'
		BNE	gl_dobackspace	; no char

gl_ctrlx:	LDX	basic_ptr	; reset pointer to input char
		STY	column_cnt	; do XON
		LDA	#$0D		; load CR

gl_chkend:	CPX	expr_stack_x	; is end of buffer reached?
		BMI	gl_savechar	; no, skip
		LDA	#7		; emit BEL character (line overflow)
		JSR	emit_char
		JMP	gl_loop		; loop

gl_savechar:	STA	0,X		; save char in buffer
		INX			; advance
		INX

gl_dobackspace:
		DEX
		STX	expr_stack_top	; save new ptr to input
		CMP	#$0D		; was char CR?
		BNE	gl_loop		; no, get another char
		JSR	IL_NL		; end of input reached
		; pop old value off stack

;------------------------------------------------------------------------------
; pop the TOS word off stack, result in IL_temp
;------------------------------------------------------------------------------

IL_SP:		JSR	pop_byte
		STA	IL_temp
		JSR	pop_byte
		STA	IL_temp+1
		RTS

;------------------------------------------------------------------------------
; IL instruction: insert BASIC line
;------------------------------------------------------------------------------

IL_IL:		JSR	swap_bp		; basicptr_save = 0x80 (input buffer)
					; basic_ptr = invalid
		JSR	find_line	; search for line with lineno from stack
					; if found: address of BASIC text in basic_ptr
					; if not: address of next line or end of program
		PHP			; save status, whether line was found
		JSR	ls_to_linestart	; adjust line back to lineno
		STA	BP_save		; save this in BP_save as well.
					; basic_ptr==BP_save is position where to enter
					; new line (if same lineno, overwrite)
		STX	BP_save+1
		LDA	IL_temp		; save lineno to be handled in LS_end
		STA	LS_end
		LDA	IL_temp+1
		STA	LS_end+1
		LDX	#0		; preload length of stored line with 0
					; for line not found (must grow)
		PLP			; restore status of find_line
		BNE	il_linenotfound	; skip if lineno not matched
					; hey, this line already exists!
		JSR	save_lineno	; save lineno where we are currently in basic_lineno
		DEX
		DEX

il_findeoln:	DEX
		JSR	fetch_basicchar
		BNE	il_findeoln	; loop until end of line
					; y now contains negative sizeof(stored line)

il_linenotfound:
		STY	basic_lineno	; clear lineno (y is 0, if line does not yet exist)
		STY	basic_lineno+1
		JSR	swap_bp		; basic_ptr = 0x80 (input buffer)
					; basicptr_save = at end of position to insert
					; (i.e. either before following line, or at end of
					; line to be grown/shrunk)
		LDA	#$0D		; calculate sizeof(input buffer)
					; load EOLN char
		CMP	(basic_ptr),Y	; start at input buffer (after line number)
					; is it eoln?
		BEQ	loc_74B		; yes, skip - this is an empty line: must delete
		INX			; no, reserve 3 bytes for lineno and CR
		INX
		INX

loc_73D:	INX			; increment for every byte in current line <> eoln
		INY
		CMP	(basic_ptr),Y	; advance and check for EOLN
		BNE	loc_73D		; loop until eoln found
					;
					; all in all, y contains the difference of line lengths:
					;  -sizeof(stored line)+sizeof(input buffer)
					; if negative: stored is longer  than new -> shrink program
					; if zero: stored is same size
					; if positive: stored is shorter than new -> grow program
		LDA	LS_end		; restore current lineno
		STA	basic_lineno
		LDA	LS_end+1
		STA	basic_lineno+1

loc_74B:	LDA	BP_save		; IL_temp = start of area to insert line
		STA	IL_temp
		LDA	BP_save+1
		STA	IL_temp+1
		CLC
		LDY	#0
		TXA			; check number of bytes
					; negative: shrink program
					; zero: nothing to move
					; positive: grow program
		BEQ	il_samesize	; same size, just copy
		BPL	loc_7AAline	; stored line is longer -> shrink
		ADC	basicptr_save	; BP_save = end_of_insert - bytes to shrink
		STA	BP_save
		LDA	basicptr_save+1
		SBC	#0
		STA	BP_save+1

il_shrink:	LDA	(basicptr_save),Y	; copy from end of insert addr to BP_save addr
		STA	(BP_save),Y
		LDX	basicptr_save
		CPX	end_prgm	; until end of program
		BNE	loc_775
		LDA	basicptr_save+1
		CMP	end_prgm+1
		BEQ	loc_7BF

loc_775:	INX			; advance
		STX	basicptr_save
		BNE	loc_77C
		INC	basicptr_save+1

loc_77C:	INC	BP_save
		BNE	il_shrink	; loop until done
		INC	BP_save+1
		BNE	il_shrink	; loop until done

loc_7AAline:	ADC	end_prgm	; make space after end of program for Y bytes
		STA	BP_save
		STA	basicptr_save
		TYA
		ADC	end_prgm+1
		STA	BP_save+1
		STA	basicptr_save+1	; basicptr_save = new end of program
		LDA	basicptr_save
		SBC	il_sp_reg
		LDA	basicptr_save+1
		SBC	il_sp_reg+1	; verify it's below top_of_RAM
		BCC	il_grow		; ok, continue
		DEC	il_pc_reg	; point back to IL instruction
		JMP	error		; overflow error

il_grow:	LDA	(end_prgm),Y	; BP_save is new end of program
		STA	(basicptr_save),Y
		LDX	end_prgm	; get byte from old end of program
		BNE	loc_7AA
		DEC	end_prgm+1	; advance back

loc_7AA:	DEC	end_prgm
		LDX	basicptr_save
		BNE	loc_7B2
		DEC	basicptr_save+1

loc_7B2:	DEX
		STX	basicptr_save
		CPX	IL_temp
		BNE	il_grow		; loop until done
		LDX	basicptr_save+1
		CPX	IL_temp+1
		BNE	il_grow		; loop until done

loc_7BF:	LDA	BP_save		; adjust new end of program
		STA	end_prgm
		LDA	BP_save+1
		STA	end_prgm+1

il_samesize:	LDA	basic_lineno	; now there is space at position for the new line
					; check lineno:  is 0 if delete
		ORA	basic_lineno+1
		BEQ	il_done		; nothing to copy (gap is already closed)
		LDA	basic_lineno
		STA	(IL_temp),Y	; start of area to insert into (the gap)
		INY
		LDA	basic_lineno+1	; store the line number into this area
		STA	(IL_temp),Y

il_moveline:	INY
		STY	LS_end		; position of gap
		JSR	fetch_basicchar	; get char from input buffer
		PHP
		LDY	LS_end		; put it into gap
		STA	(IL_temp),Y
		PLP
		BNE	il_moveline	; until EOLN

il_done:	JMP	restart_il	; re-enter BASIC loop

;------------------------------------------------------------------------------
; IL instruction: divide
;------------------------------------------------------------------------------

IL_DV:		JSR	expr_check_4bytes	; validate 2 args on stack
		LDA	3,X		; high byte dividend
		AND	#$80		; sign
		BEQ	loc_7F2
		LDA	#$0FF		; A=0xFF if sign=1, 0x00 if sign=0

loc_7F2:	STA	IL_temp		; sign extend dividend into 32bit (IL_temp=high word)
		STA	IL_temp+1
		PHA			; if negative, subtract 1 from dividend
					; 0x0000...0x7fff stays positive
					; 0x8000 becomes positive
					; 0x8001...0xffff stays negative
		ADC	2,X
		STA	2,X
		PLA
		PHA
		ADC	3,X
		STA	3,X
		PLA
		EOR	1,X		; compare with sign of divisor
		STA	X_save+1
		BPL	loc_80B		; if different sign, complement divisor
					; i.e. NEG/NEG -> do nothing
					;      NEG/POS -> NEG/NEG + lead_zero <0
					;      POS/NEG -> POS/POS + lead_zero <0
					;      POS/POS -> do nothing
		JSR	negate

loc_80B:	LDY	#$11		; loop counter (16+1 iterations)
		LDA	0,X		; is divisor = 0?
		ORA	1,X
		BNE	dv_loop		; no, do division
		JMP	error

dv_loop:	SEC
		LDA	IL_temp		; subtract divisor from 32bit dividend
		SBC	0,X
		PHA			; remember result
		LDA	IL_temp+1
		SBC	1,X
		PHA
		EOR	IL_temp+1
		BMI	dv_smaller	; subtract result was <0 ?
		PLA			; no, can subtract, remember a 1 bit (sec)
		STA	IL_temp+1	; and store new result
		PLA
		STA	IL_temp
		SEC
		JMP	dv_shift

dv_smaller:	PLA			; discard subtraction
		PLA
		CLC			; remember 0 bit

dv_shift:	ROL	2,X		; shift 32bit dividend left
		ROL	3,X
		ROL	IL_temp		; shift in result bit into lowest bit of dividend
		ROL	IL_temp+1
		DEY			; decrement loop
		BNE	dv_loop		; subtract divisor from  32bit dividend
		LDA	X_save+1
		BPL	locret_84E

;------------------------------------------------------------------------------
; IL instruction: negate top of	stack
;------------------------------------------------------------------------------

IL_NE:		LDX	expr_stack_x	; point to TOS

negate:		SEC
		TYA
		SBC	0,X
		STA	0,X
		TYA
		SBC	1,X
		STA	1,X

locret_84E:	RTS

;------------------------------------------------------------------------------
; IL instruction: subtract TOS from NOS	-> NOS
;------------------------------------------------------------------------------

IL_SU:		JSR	IL_NE	; negate TOS. [ A-B is A+(-B) ]

;------------------------------------------------------------------------------
; IL instruction: add TOS and NOS -> NOS
;------------------------------------------------------------------------------

IL_AD:		JSR	expr_check_4bytes
		LDA	0,X	; add TOS and NOS
		ADC	2,X
		STA	2,X
		LDA	1,X
		ADC	3,X
		STA	3,X
		RTS

;------------------------------------------------------------------------------
; multiply TOS with NOS	-> NOS
; I think this this routine is broken for negative numbers
;------------------------------------------------------------------------------

IL_MP:		JSR	expr_check_4bytes	; validate 2 args
		LDY	#$10		; bit count (16 bits)
		LDA	2,X
		STA	IL_temp
		LDA	3,X
		STA	IL_temp+1

mp_loop:	ASL	2,X		; shift 1 bit left
		ROL	3,X
		ROL	IL_temp		; shift 1st operand
		ROL	IL_temp+1
		BCC	mp_notadd	; is top bit = 1?
		CLC			; yes, add 2nd operand
		LDA	2,X
		ADC	0,X
		STA	2,X
		LDA	3,X
		ADC	1,X
		STA	3,X

mp_notadd:	DEY			; decrement counter
		BNE	mp_loop		; loop 16 times
		RTS

;------------------------------------------------------------------------------
; IL instruction: fetch	variable
;------------------------------------------------------------------------------

IL_FV:		JSR	pop_byte	; get byte (variable address into zero page)
		TAX
		LDA	0,X		; make pointer into var table
		LDY	1,X
		DEC	expr_stack_x
		LDX	expr_stack_x
		STY	0,X
		JMP	expr_push_byte	; push it onto expr_stack

;------------------------------------------------------------------------------
; IL instruction: save variable
;------------------------------------------------------------------------------

IL_SV:		LDX	#(rnd_seed-3)	; $7D
		JSR	expr_check_nbytes	; validate stack contains var index byte
					; and data word. drop single byte
		LDA	1,X		; get low byte of data
		PHA
		LDA	0,X		; get high byte of data in A
		PHA
		JSR	pop_byte
		TAX
		PLA
		STA	0,X		; save into variable
		PLA
		STA	1,X
		RTS

;------------------------------------------------------------------------------
; IL instruction return	from IL	call
;------------------------------------------------------------------------------

IL_RT:		JSR	loc_8FD		; get saved IL PC address
		LDA	IL_temp		; restore it to IL_PC
		STA	il_pc_reg
		LDA	IL_temp+1
		STA	il_pc_reg+1
		RTS

;------------------------------------------------------------------------------
; IL instruction save BASIC pointer
;------------------------------------------------------------------------------

IL_SB:		LDX	#basic_ptr	; get address of basic pointer
		BNE	loc_8C3		; continue in IL_RB common code

;------------------------------------------------------------------------------
; IL instruction: restore BASIC	pointer
;------------------------------------------------------------------------------

IL_RB:		LDX	#basicptr_save
loc_8C3:	LDA	0,X		; is it into the input line area?
		CMP	#rnd_seed
		BCS	swap_bp
		LDA	1,X
		BNE	swap_bp		; no, do swap with save location
		LDA	basic_ptr
		STA	basicptr_save
		LDA	basic_ptr+1
		STA	basicptr_save+1
		RTS

swap_bp:	LDA	basic_ptr	; get basic pointer
		LDY	basicptr_save	; move saved pointer to basic ptr
		STY	basic_ptr
		STA	basicptr_save
		LDA	basic_ptr+1
		LDY	basicptr_save+1
		STY	basic_ptr+1
		STA	basicptr_save+1
		LDY	#0
		RTS

;------------------------------------------------------------------------------
; IL instruction gosub
;------------------------------------------------------------------------------

IL_GS:		LDA	basic_lineno	; get line number of GOSUB
		STA	IL_temp		; store it in temp
		LDA	basic_lineno+1
		STA	IL_temp+1
		JSR	handle_30a

loc_8F4:	LDA	il_sp_reg
		STA	top_of_stack
		LDA	il_sp_reg+1
		STA	top_of_stack+1

locret_8FC:	RTS

;---------------------------------------

loc_8FD:	LDA	(il_sp_reg),Y
		STA	IL_temp
		JSR	loc_908
		LDA	(il_sp_reg),Y
		STA	IL_temp+1

loc_908:	INC	il_sp_reg
		BNE	loc_90E
		INC	il_sp_reg+1

loc_90E:	LDA	end_ram
		CMP	il_sp_reg
		LDA	end_ram+1
		SBC	il_sp_reg+1
		BCS	locret_8FC
		JMP	error

;------------------------------------------------------------------------------
; IL instruction US: machine language subroutine call
;------------------------------------------------------------------------------

IL_US:		JSR	us_do		; call machine language	routine
		STA	IL_temp
		TYA
		JMP	loc_582

us_do:		JSR	IL_SP
		LDA	IL_temp
		STA	LS_end
		JSR	IL_SP
		LDA	IL_temp+1
		STA	LS_end+1
		LDY	IL_temp
		JSR	IL_SP
		LDX	LS_end+1
		LDA	LS_end
		CLC
		JMP	(IL_temp)

;------------------------------------------------------------------------------
; IL instruction push word
;------------------------------------------------------------------------------

IL_LN:		JSR	IL_LB

;------------------------------------------------------------------------------
; IL instruction push byte
;------------------------------------------------------------------------------

IL_LB:		JSR	fetch_il_op	; get next byte from sequence
		JMP	expr_push_byte	; push single byte

loc_948:	STX	basic_ptr+1
		CPX	#0
		RTS

;------------------------------------------------------------------------------
; These two RESERVED entry points are for code that
; does not seem to get called.  Need more research.
;------------------------------------------------------------------------------

ILRES2:		LDY	#2

;---------------------------------------

ILRES1:		STY	IL_temp
		LDY	#$0B
		STY	IL_temp+1
		LDY	#0
		LDA	(IL_temp),Y
		CMP	#8
		BNE	locret_960
		JMP	il_goto

locret_960:	RTS

;----------------------------------------
; to crlf
; Subroutine to decide which pad characters to print

out_v_padding:	JSR	out_v		; Entry point with a character to print first
emit_nul_padding:
		LDA	#$0FF		; Set pad to $FF
		BIT	PCC		; Check if the pad flag is on
		BMI	loc_96D		; Skip it if not
		LDA	#0		; set pad to $00
loc_96D:	JMP	out_v		; print it

;------------------------------------------------------------------------------
; The IL interpreter
; five changes to original IL from TBEK.txt
;------------------------------------------------------------------------------
;
;	orig							here
;STRT: PC ":\17"       ;COLON, X-ON                     --> STRT: PC ">\0"   
;I3:   PC "? \17"      ;LINE IS EMPTY; TYPE PROMPT ^Q       I3:   PC "? \0"  
;      LB 3            ; <=                                       LB 93h    why??  
;R3:   LB 1            ; <                                  R3:   LB 91h    why?? 
;      LB 5            ; ><                                       LB 95h    why?? 
;

ILTBL    fcb $24, $3E, $80, $27, $10, $E1, $59, $C5, $2A, $56, $10, $11, $2C, $8B, $4C
	 fcb $45, $D4, $A0, $80, $BD, $30, $BC, $E0, $13, $1D, $94, $47, $CF, $88, $54
	 fcb $CF, $30, $BC, $E0, $10, $11, $16, $80, $53, $55, $C2, $30, $BC, $E0, $14
	 fcb $16, $90, $50, $D2, $83, $49, $4E, $D4, $E5, $71, $88, $BB, $E1, $1D, $8F
	 fcb $A2, $21, $58, $6F, $83, $AC, $22, $55, $83, $BA, $24, $93, $E0, $23, $1D
	 fcb $30, $BC, $20, $48, $91, $49, $C6, $30, $BC, $31, $34, $30, $BC, $84, $54
	 fcb $48, $45, $CE, $1C, $1D, $38, $0D, $9A, $49, $4E, $50, $55, $D4, $A0, $10
	 fcb $E7, $24, $3F, $20, $80, $27, $E1, $59, $81, $AC, $30, $BC, $13, $11, $82
	 fcb $AC, $4D, $E0, $1D, $89, $52, $45, $54, $55, $52, $CE, $E0, $15, $1D, $85
	 fcb $45, $4E, $C4, $E0, $2D, $98, $4C, $49, $53, $D4, $EC, $24, $00, $00, $00
	 fcb $00, $0A, $80, $1F, $24, $93, $23, $1D, $30, $BC, $E1, $50, $80, $AC, $59
	 fcb $85, $52, $55, $CE, $38, $0A, $86, $43, $4C, $45, $41, $D2, $2B, $84, $52
	 fcb $45, $CD, $1D, $A0, $80, $BD, $38, $14, $85, $AD, $30, $D3, $17, $64, $81
	 fcb $AB, $30, $D3, $85, $AB, $30, $D3, $18, $5A, $85, $AD, $30, $D3, $19, $54
	 fcb $2F, $30, $E2, $85, $AA, $30, $E2, $1A, $5A, $85, $AF, $30, $E2, $1B, $54
	 fcb $2F, $98, $52, $4E, $C4, $0A, $80, $80, $12, $0A, $09, $29, $1A, $0A, $1A
	 fcb $85, $18, $13, $09, $80, $12, $01, $0B, $31, $30, $61, $72, $0B, $04, $02
	 fcb $03, $05, $03, $1B, $1A, $19, $0B, $09, $06, $0A, $00, $00, $1C, $17, $2F
	 fcb $8F, $55, $53, $D2, $80, $A8, $30, $BC, $31, $2A, $31, $2A, $80, $A9, $2E
	 fcb $2F, $A2, $12, $2F, $C1, $2F, $80, $A8, $30, $BC, $80, $A9, $2F, $83, $AC
	 fcb $38, $BC, $0B, $2F, $80, $A8, $52, $2F, $84, $BD, $09, $02, $2F, $8E, $BC
	 fcb $84, $BD, $09, $93, $2F, $84, $BE, $09, $05, $2F, $09, $91, $2F, $80, $BE
	 fcb $84, $BD, $09, $06, $2F, $84, $BC, $09, $95, $2F, $09, $04, $2F
;
; End of Tiny Basic

	end
